@@ -29,7 +29,7 @@
 #include "gst-report-hook.h"
 #include "gst-platform.h"
 #include "gst-ui.h"
-#include "gst-su.h"
+#include "gst-auth.h"
 #include "gst-xml.h"
 #include "gst-marshal.h"
 
@@ -51,7 +51,10 @@
 #include <sys/wait.h>
 #include <signal.h>
 
+#define BUFFER_SIZE 2048
 #define GST_DEBUG 1
+#define GST_TOOL_EOR "<!-- GST: end of request -->"
+
 /* Define this if you want the frontend to run the backend under strace */
 /*#define GST_DEBUG_STRACE_BACKEND*/
 
@@ -65,6 +68,8 @@ GdkPixbuf *suse;
 GdkPixbuf *freebsd;
 GdkPixbuf *gentoo;
 GdkPixbuf *pld;
+GdkPixbuf *openna;
+GdkPixbuf *fedora;
 
 enum {
 	BOGUS,
@@ -82,16 +87,8 @@ enum {
 	PLATFORM_LIST_COL_LAST
 };
 
-static enum {
-	ROOT_ACCESS_NONE,
-	ROOT_ACCESS_SIMULATED,
-	ROOT_ACCESS_SIMULATED_DISABLED,
-	ROOT_ACCESS_REAL
-} root_access = ROOT_ACCESS_NONE;
-
 static GtkObjectClass *parent_class;
 static gint gsttool_signals [LAST_SIGNAL] = { 0 };
-static const gchar *GST_TOOL_EOR = "\n<!-- GST: end of request -->\n";
 
 static void     gst_tool_idle_run_directives_remove (GstTool *tool);
 static void     gst_tool_idle_run_directives_add    (GstTool *tool);
@@ -105,9 +102,9 @@ static void report_dispatch (GstTool *tool);
 static GstReportHookEntry common_report_hooks[] = {
 /*        key                 function                    type                      allow_repeat user_data */
 	{ "end",              report_finished_cb,         GST_REPORT_HOOK_LOADSAVE, TRUE,        NULL },
-	{ "platform_success", platform_set_current_cb,    GST_REPORT_HOOK_SAVE,     FALSE,       NULL }, 
-	{ "platform_unsup",   platform_unsupported_cb,    GST_REPORT_HOOK_LOADSAVE, FALSE,       NULL },
-	{ "platform_undet",   platform_unsupported_cb,    GST_REPORT_HOOK_LOADSAVE, FALSE,       NULL },
+	{ "platform_success", platform_set_current_cb,    GST_REPORT_HOOK_SAVE,     TRUE,        NULL }, 
+	{ "platform_unsup",   platform_unsupported_cb,    GST_REPORT_HOOK_LOADSAVE, TRUE,        NULL },
+	{ "platform_undet",   platform_unsupported_cb,    GST_REPORT_HOOK_LOADSAVE, TRUE,        NULL },
 	{ NULL,               NULL,                       -1,                       FALSE,       NULL }
 };
 
@@ -130,6 +127,7 @@ on_platform_list_selection_changed (GtkTreeSelection *selection, gpointer data)
 	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		gtk_tree_model_get (model, &iter, PLATFORM_LIST_COL_PLATFORM, &platform, -1);
 		tool->current_platform = gst_platform_dup (platform);
+
 		selected = TRUE;
 	} else {
 		tool->current_platform = NULL;
@@ -263,32 +261,9 @@ gst_tool_get_dialog (GstTool *tool)
 gboolean
 gst_tool_get_access (GstTool *tool)
 {
-	return root_access != ROOT_ACCESS_NONE;
+	return tool->root_access != ROOT_ACCESS_NONE;
 }
 
-#if 0
-static void
-timeout_cb (gpointer data)
-{
-	GstTool *tool;
-
-	tool = GST_TOOL (data);
-
-	tool->timeout_done = TRUE;
-
-	if (tool->timeout_done && !tool->report_list_visible)
-		gtk_main_quit ();
-}
-
-static void
-set_arrow (GstTool *tool, GtkArrowType arrow)
-{
-	gtk_notebook_set_page (GTK_NOTEBOOK (tool->report_notebook),
-			       arrow == GTK_ARROW_UP ? 0 : 1);
-}
-
-#endif
-
 static void
 report_clear_lines (GstTool *tool)
 {
@@ -313,23 +288,6 @@ report_dispatch (GstTool *tool)
 {
 	GSList *list;
 	GstReportLine *rline;
-#if 0
-	GtkAdjustment *vadj;
-	char *report_text[] = {
-		NULL,
-		NULL
-	};
-
-	if (!GTK_IS_SCROLLED_WINDOW (tool->report_scrolled)) {
-		g_warning ("tool->report_scrolled is not a GtkScrolledWindow\n");
-		tool->report_dispatch_pending = FALSE;
-		tool->report_finished = TRUE;
-		return;
-	}
-	
-	vadj = gtk_scrolled_window_get_vadjustment (
-		GTK_SCROLLED_WINDOW (tool->report_scrolled));
-#endif
 
 	for (list = tool->report_line_list; list; list = g_slist_next (list))
 	{
@@ -338,36 +296,6 @@ report_dispatch (GstTool *tool)
 		if (gst_report_line_get_handled (rline))
 			continue;
 
-#if 0
-		if (!strcmp (gst_report_line_get_key (rline), "progress")) {
-			/* Progress update */
-
-			gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress),
-						     atoi (gst_report_line_get_message (rline)) / 100.0);
-		} else {
-			gboolean scroll;
-			
-			/* Report line */
-			
-			gtk_entry_set_text (GTK_ENTRY (tool->report_entry),
-					    gst_report_line_get_message (rline));
-			
-			if (vadj->value >= vadj->upper - vadj->page_size)
-				scroll = TRUE;
-			else
-				scroll = FALSE;
-
-			report_text [1] = (char *) gst_report_line_get_message (rline);
-
-			gtk_clist_append (GTK_CLIST (tool->report_list), report_text);
-
-			if (scroll)
-				gtk_adjustment_set_value (vadj, vadj->upper - vadj->page_size);
-
-			gst_tool_invoke_report_hooks (tool, tool->report_hook_type, rline);
-		}
-#endif
-
 		if (strcmp (gst_report_line_get_key (rline), "progress"))
 			gst_tool_invoke_report_hooks (tool, tool->report_hook_type, rline);
 		
@@ -386,8 +314,8 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 {
 	GstTool *tool;
 	GstReportLine *rline;
-	gchar buffer [512];
-	gint n, i = 0;
+	gchar *buffer;
+	gint i = 0;
 
 	tool = GST_TOOL (data);
 	if (tool->input_block)
@@ -396,18 +324,16 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 	if (!tool->line)
 		tool->line = g_string_new ("");
 
-	while ((n = read (fd, buffer, sizeof (buffer) - 1)) != 0) {
-		buffer [n] = 0;
-		
-		for (i = 0; (i < n); i++) {
+	while ((buffer = gst_tool_read_from_backend (tool))) {
+		for (i = 0; (i < strlen (buffer)); i++) {
 			gchar c = buffer [i];
-			
+
 			if (c == '\n') {
 				/* End of line */
-				
+
 				/* Report line; add to list */
 				rline = gst_report_line_new_from_string (tool->line->str);
-				
+
 				if (rline)
 					tool->report_line_list = g_slist_append (
 						tool->report_line_list, rline);
@@ -428,17 +354,11 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 		}
 	}
 
- full_break:
-
-	if (n <= 0) {
-		/* Zero-length read; pipe closed unexpectedly */
-
-		tool->report_finished = TRUE;
-	}
-
+full_break:
 	if (tool->report_finished) {
-		if (n > 0)
+		if ((strlen (buffer) - i) > 0)
 			tool->xml_document = g_string_new (&buffer [i]);
+		
 		g_string_free (tool->line, TRUE);
 		tool->line = NULL;
 	}
@@ -464,9 +384,6 @@ report_window_close_cb (GtkWidget *window, GdkEventAny *ev, gpointer data)
 static void
 report_progress (GstTool *tool, const gchar *label)
 {
-#if 0
-	gint cb_id;
-#endif	
 	tool->timeout_done = FALSE;
 	tool->report_list_visible = FALSE;
 	tool->report_finished = FALSE;
@@ -475,11 +392,6 @@ report_progress (GstTool *tool, const gchar *label)
 	gst_tool_clear_supported_platforms (tool);
 	report_clear_lines (tool);
 
-#if 0
-	set_arrow (tool, GTK_ARROW_DOWN);
-	gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress), 0.0);
-#endif
-
 	if (label) {
 		gtk_label_set_text (GTK_LABEL (tool->report_label), label);
 		gtk_widget_show_all (tool->report_window);
@@ -492,10 +404,9 @@ report_progress (GstTool *tool, const gchar *label)
 			usleep (1);
 			gtk_main_iteration ();
 		}
-		sleep (1);
 	}
 
-	tool->input_id = gtk_input_add_full (tool->backend_read_fd, GDK_INPUT_READ,
+	tool->input_id = gtk_input_add_full (tool->backend_master_fd, GDK_INPUT_READ,
 					     report_progress_tick, NULL, tool, NULL);
 	tool->input_block = FALSE;
 
@@ -503,20 +414,6 @@ report_progress (GstTool *tool, const gchar *label)
 
 	if (tool->input_id)
 		gtk_input_remove (tool->input_id);
-
-#if 0
-	if (!tool->run_again)
-	{
-		/* Set progress to 100% and wait a bit before closing display */
-		gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress), 1.0);
-
-		cb_id = gtk_timeout_add (1500, (GtkFunction) timeout_cb, tool);
-		gtk_main ();
-		gtk_timeout_remove (cb_id);
-	}
-
-	gtk_clist_clear (GTK_CLIST (tool->report_list));
-#endif
 }
 
 static GSList *
@@ -555,6 +452,8 @@ gst_tool_create_distro_images (void)
 	freebsd = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/freebsd.png", NULL);
 	gentoo = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/gentoo.png", NULL);
 	pld = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/pld.png", NULL);
+	openna = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/openna.png", NULL);
+	fedora = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/fedora.png", NULL);
 }
 
 static void
@@ -570,6 +469,8 @@ gst_tool_fill_platform_tree (GstTool *tool)
 
 	/* Fill in the platform GtkTreeView */
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tool->platform_list));
+
+	gtk_tree_store_clear (GTK_TREE_STORE (model));
 	
 	tool->supported_platforms_list = gst_tool_get_supported_platforms (tool);
 	
@@ -586,6 +487,21 @@ gst_tool_fill_platform_tree (GstTool *tool)
 	}
 }
 
+static void
+gst_tool_fill_remote_hosts_list (GstTool *tool, gchar *hosts_list[])
+{
+	gchar **host;
+	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (tool->remote_hosts_list));
+	GtkTreeIter iter;
+
+	for (host = hosts_list; *host != NULL; host++) {
+		gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
+		gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
+				    0, *host,
+				    -1);
+	}
+}
+
 static void
 gst_tool_run_platform_dialog (GstTool *tool)
 {
@@ -595,16 +511,18 @@ gst_tool_run_platform_dialog (GstTool *tool)
 	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
 	
 	result = gtk_dialog_run (GTK_DIALOG (tool->platform_dialog));
-	gtk_widget_destroy (tool->platform_dialog);
+	gtk_widget_hide (tool->platform_dialog);
+
 	if (result != GTK_RESPONSE_OK)
 		exit (0);
 }
 
-static void
+void
 gst_tool_process_startup (GstTool *tool)
 {
 	/* let's process those startup reports. */
 	tool->report_hook_type = GST_REPORT_HOOK_LOAD;
+
 	report_progress (tool, NULL);
 
 	/* stable version users should pass here only once,
@@ -612,72 +530,15 @@ gst_tool_process_startup (GstTool *tool)
 	while (tool->run_again) {
 		gst_tool_run_platform_dialog (tool);
 		g_assert (tool->current_platform);
-		
-		if (root_access == ROOT_ACCESS_SIMULATED)
-			root_access = ROOT_ACCESS_SIMULATED_DISABLED;
-		
+
+		if (tool->root_access == ROOT_ACCESS_SIMULATED)
+			tool->root_access = ROOT_ACCESS_SIMULATED_DISABLED;
+
 		gst_tool_run_set_directive (tool, NULL, NULL, "platform_set",
 					    gst_platform_get_key (tool->current_platform), NULL);
 
-		if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
-			root_access = ROOT_ACCESS_SIMULATED;
-	}
-}
-
-static void
-gst_tool_init_backend (GstTool *tool)
-{
-	int fd_read [2], fd_write [2];
-
-	g_return_if_fail (tool != NULL);
-	g_return_if_fail (GST_IS_TOOL (tool));
-	g_return_if_fail (tool->script_path != NULL);
-	g_return_if_fail (tool->backend_pid < 0);
-	
-	pipe (fd_read);
-	pipe (fd_write);
-
-	tool->backend_pid = fork ();
-	if (tool->backend_pid < 0) {
-		g_warning ("Unable to fork.");
-		tool->backend_pid = -1;
-		return;
-	} else if (tool->backend_pid) {
-		/* Parent */
-
-		close (fd_read  [1]); /* Close writing end of read pipe */
-		close (fd_write [0]); /* Close reading end of write pipe */
-
-		tool->backend_read_fd  = fd_read  [0];
-		tool->backend_write_fd = fd_write [1];
-
-		fcntl (tool->backend_read_fd, F_SETFL, 0);  /* Let's block */
-
-		gst_tool_process_startup (tool);
-	} else {
-		/* Child */
-
-		/* For the child, things are upside-down */
-		close (fd_read  [0]); /* Close reading end of read pipe */
-		close (fd_write [1]); /* Close writing end of write pipe */
-
-		dup2 (fd_read  [1], STDOUT_FILENO);
-		dup2 (fd_write [0],  STDIN_FILENO);
-
-		if (tool->current_platform)
-			execl (tool->script_path, tool->script_path,
-			       "--progress", "--report", "--platform",
-			       gst_platform_get_key (tool->current_platform), NULL);
-		else
-#ifndef GST_DEBUG_STRACE_BACKEND
-			execl (tool->script_path, tool->script_path, "--progress",
-			       "--report", NULL);
-#else		
-		execl ("/usr/bin/strace", "/usr/bin/strace", "-f", tool->script_path, "--progress",
-		       "--report", NULL);
-#endif					
-		
-		g_error ("Unable to run backend: %s", tool->script_path);
+		if (tool->root_access == ROOT_ACCESS_SIMULATED_DISABLED)
+			tool->root_access = ROOT_ACCESS_SIMULATED;
 	}
 }
 
@@ -721,24 +582,46 @@ static void
 gst_tool_kill_backend_cb (GstDirectiveEntry *entry)
 {
 	GstTool *tool = entry->tool;
-	
+
+	/* removes the g_timeout that's waiting for the backend to die */
+	g_source_remove (tool->timeout_id);
+
 	/* The backend was never called. No problem! */
 	/* For further reference check http://www.xs4all.nl/~pjbrink/apekool/050/045-en.html */
 	if (tool->backend_pid > 0) {
-		if (root_access == ROOT_ACCESS_SIMULATED)
-			root_access = ROOT_ACCESS_SIMULATED_DISABLED;
-		
+		if (tool->root_access == ROOT_ACCESS_SIMULATED)
+			tool->root_access = ROOT_ACCESS_SIMULATED_DISABLED;
+
 		gst_tool_run_set_directive (tool, NULL, NULL, "end", NULL);
 		
-		if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
-			root_access = ROOT_ACCESS_SIMULATED;
-	
+		if (tool->root_access == ROOT_ACCESS_SIMULATED_DISABLED)
+			tool->root_access = ROOT_ACCESS_SIMULATED;
+
 		waitpid (tool->backend_pid, NULL, 0);
 	}
-	
-	gtk_signal_emit_by_name (GTK_OBJECT (tool), "destroy");
 }
 
+static void
+gst_tool_kill_tool_cb (GstDirectiveEntry *entry)
+{
+	GstTool *tool = entry->tool;
+
+	gst_tool_kill_backend_cb (entry);
+	
+	g_signal_emit_by_name (G_OBJECT (tool), "destroy");
+}
+
+/* kills the whole tool */
+static void
+gst_tool_kill_tool (GstTool *tool, gpointer data)
+{
+	g_return_if_fail (tool != NULL);
+	g_return_if_fail (GST_IS_TOOL (tool));
+
+	gst_tool_queue_directive (tool, gst_tool_kill_tool_cb, NULL, NULL, NULL, "end");
+}
+
+/* only kills the backend */
 static void
 gst_tool_kill_backend (GstTool *tool, gpointer data)
 {
@@ -751,20 +634,18 @@ gst_tool_kill_backend (GstTool *tool, gpointer data)
 static gboolean
 gst_tool_end_of_request (GString *string)
 {
-	gchar *str;
-	gint len, eorlen, i;
+	gchar *str, *eorpos;
+	gint len, i;
 
 	str = string->str;
 	len = strlen (str);
-	eorlen = strlen (GST_TOOL_EOR);
-	if (len < eorlen)
+
+	eorpos = g_strrstr (str, GST_TOOL_EOR);
+
+	if (eorpos == NULL)
 		return FALSE;
 
-	for (i = 0; i < eorlen; i++)
-		if (GST_TOOL_EOR[eorlen - i - 1] != str[len - i - 1])
-			return FALSE;
-
-	g_string_truncate (string, len - eorlen + 1);
+	g_string_truncate (string, strlen (str) - strlen (eorpos));
 
 	return TRUE;
 }
@@ -772,31 +653,17 @@ gst_tool_end_of_request (GString *string)
 static xmlDoc *
 gst_tool_read_xml_from_backend (GstTool *tool)
 {
-	char buffer [4096];
+	gchar *buffer;
 	int t;
 	xmlDoc *xml;
 
 	if (!tool->xml_document)
 		return NULL;
-	
-	fcntl (tool->backend_read_fd, F_SETFL, O_NONBLOCK);
 
 	while (! gst_tool_end_of_request (tool->xml_document)) {
-		while (gtk_events_pending ())
-			gtk_main_iteration ();
-
-		t = read (tool->backend_read_fd, buffer, sizeof (buffer) - 1);
-
-		if (t == 0)
-			break;
-		if (t == -1)
-			t = 0;
-
-		buffer [t] = 0;
+		buffer = gst_tool_read_from_backend (tool);
 		g_string_append (tool->xml_document, buffer);
 	}
-	
-	fcntl (tool->backend_read_fd, F_SETFL, 0);
 
 	if (tool->xml_document->str[0] == '<') {
 		xml = xmlParseDoc (tool->xml_document->str);
@@ -875,16 +742,14 @@ static void
 gst_tool_send_directive (GstTool *tool, const gchar *directive, va_list ap)
 {
 	GString *directive_line;
-	FILE *f;
+	gchar *buffer;
 
 	g_return_if_fail (tool->backend_pid >= 0);
 
        	directive_line = gst_tool_join_directive (directive, ap);
 	g_string_append_c (directive_line, '\n');
-	
-	f = fdopen (dup (tool->backend_write_fd), "w");
-	fprintf (f, directive_line->str);
-	fclose (f);
+
+	gst_tool_write_to_backend (tool, directive_line->str);
 
 	g_string_free (directive_line, TRUE);
 }
@@ -898,9 +763,6 @@ gst_tool_run_get_directive_va (GstTool *tool, const gchar *report_sign, const gc
 	g_return_val_if_fail (GST_IS_TOOL (tool), NULL);
 	g_return_val_if_fail (tool->directive_running == FALSE, NULL);
 
-	if (tool->backend_pid < 0)
-		gst_tool_init_backend (tool);
-	
 	tool->directive_running = TRUE;
 	gst_tool_idle_run_directives_remove (tool);
 
@@ -923,8 +785,8 @@ gst_tool_run_get_directive_va (GstTool *tool, const gchar *report_sign, const gc
 	if (location_id == NULL)
 		report_progress (tool, report_sign? _(report_sign): NULL);
 
-	xml = gst_tool_read_xml_from_backend (tool);
-	
+	xml = gst_tool_read_xml_from_backend(tool);
+
 	tool->directive_running = FALSE;
 	gst_tool_idle_run_directives_add (tool);
 	
@@ -944,21 +806,19 @@ static xmlDoc *
 gst_tool_run_set_directive_va (GstTool *tool, xmlDoc *xml,
 			       const gchar *report_sign, const gchar *directive, va_list ap)
 {
-	FILE *f;
 	xmlDoc *xml_out;
-	
+	int n;
+	gchar buf;
+
 	g_return_val_if_fail (tool != NULL, NULL);
 	g_return_val_if_fail (GST_IS_TOOL (tool), NULL);
 	g_return_val_if_fail (tool->directive_running == FALSE, NULL);
 
-	if (tool->backend_pid < 0)
-		gst_tool_init_backend (tool);
-
 	tool->directive_running = TRUE;
 	gst_tool_idle_run_directives_remove (tool);
 
 	/* don't actually run if we are just pretending */
-	if (root_access == ROOT_ACCESS_SIMULATED) {
+	if (tool->root_access == ROOT_ACCESS_SIMULATED) {
 		g_warning (_("Skipping %s directive..."), directive);
 		tool->directive_running = FALSE;
 		return NULL;
@@ -967,12 +827,13 @@ gst_tool_run_set_directive_va (GstTool *tool, xmlDoc *xml,
 	gst_tool_send_directive (tool, directive, ap);
 
 	if (xml) {
-		f = fdopen (dup (tool->backend_write_fd), "w");
-		xmlDocDump (f, xml);
-		fprintf (f, GST_TOOL_EOR);
-		fclose (f);
+		gst_tool_write_xml_to_backend (tool, xml);
+		gst_tool_write_to_backend (tool, "\n");
+		gst_tool_write_to_backend (tool, GST_TOOL_EOR);
+		gst_tool_write_to_backend (tool, "\n");
 	}
 
+
 	/* FIXME: Instead of doing the following, we should pass around a value describing
 	 * tool I/O mode (as opposed to a string to report_progress ()). */
 	tool->report_hook_type = GST_REPORT_HOOK_SAVE;
@@ -980,6 +841,7 @@ gst_tool_run_set_directive_va (GstTool *tool, xmlDoc *xml,
 	if (location_id == NULL)
 		report_progress (tool, report_sign? _(report_sign): NULL);
 
+
 	/* This is tipicaly to just read the end of request string,
 	   but a set directive may return some XML too. */
 	xml_out = gst_tool_read_xml_from_backend (tool);
@@ -1009,15 +871,17 @@ gst_tool_load (GstTool *tool)
   
 	if (tool->config) {
 		xmlFreeDoc (tool->config);
+		xmlFreeDoc (tool->original_config);
 		tool->config = NULL;
+		tool->original_config = NULL;
 	}
 
 	if (tool->run_again)
 		return TRUE;
 
 	if (location_id == NULL) {
-		tool->config = gst_tool_run_get_directive
-			(tool, _("Scanning your system configuration."), "get", NULL);
+		tool->config = gst_tool_run_get_directive (tool, NULL, "get", NULL);
+		tool->original_config = xmlCopyDoc (tool->config, 1);
 	} else {
 #ifndef GST_HAVE_ARCHIVER
 		g_assert_not_reached ();
@@ -1066,18 +930,8 @@ gst_tool_load (GstTool *tool)
 	return tool->config != NULL;
 }
 
-#if 0
-static void
-gst_tool_save_directive_callback (GstDirectiveEntry *entry)
-{
-	gst_tool_run_set_directive_va (entry->tool, entry->in_xml, entry->report_sign,
-				       entry->directive, entry->ap);
-	gst_dialog_thaw_visible (entry->tool->main_dialog);
-}
-#endif
-
 gboolean
-gst_tool_save (GstTool *tool)
+gst_tool_save (GstTool *tool, gboolean restore)
 {
 #ifdef GST_HAVE_ARCHIVER
 	CORBA_Environment ev;
@@ -1088,15 +942,14 @@ gst_tool_save (GstTool *tool)
 
 	g_return_val_if_fail (tool != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_TOOL (tool), FALSE);
-	g_return_val_if_fail (root_access != ROOT_ACCESS_NONE, FALSE);
+	g_return_val_if_fail (tool->root_access != ROOT_ACCESS_NONE, FALSE);
 
 	gst_dialog_freeze_visible (tool->main_dialog);
 
 	g_signal_emit (GTK_OBJECT (tool), gsttool_signals[FILL_XML], 0);
-
 #ifdef GST_DEBUG
 	/* don't actually save if we are just pretending */
-	if (root_access == ROOT_ACCESS_SIMULATED) {
+	if (tool->root_access == ROOT_ACCESS_SIMULATED) {
 		g_warning (_("Skipping actual save..."));
 		return TRUE;
 	}
@@ -1138,9 +991,13 @@ gst_tool_save (GstTool *tool)
 
 #endif
 	
-	if (location_id == NULL)
-		gst_tool_run_set_directive (tool, tool->config, _("Updating your system configuration."),
-					    "set", NULL);
+	if (location_id == NULL) {
+		if (restore)
+			gst_tool_run_set_directive (tool, tool->original_config, NULL, "set", NULL);
+		else
+			gst_tool_run_set_directive (tool, tool->config, NULL, "set", NULL);
+	}
+	
 	gst_dialog_thaw_visible (tool->main_dialog);
 
 	return TRUE;  /* FIXME: Determine if it really worked. */
@@ -1149,7 +1006,13 @@ gst_tool_save (GstTool *tool)
 void
 gst_tool_save_cb (GtkWidget *w, GstTool *tool)
 {
-	gst_tool_save (tool);
+	gst_tool_save (tool, FALSE);
+}
+
+void
+gst_tool_restore_cb (GtkWidget *w, GstTool *tool)
+{
+	gst_tool_save (tool, TRUE);
 }
 
 void
@@ -1170,19 +1033,45 @@ gst_tool_load_try (GstTool *tool)
 	}
 }
 
-void
-gst_tool_main (GstTool *tool, gboolean no_main_loop)
+static void
+tool_main_do (GstTool *tool, gboolean no_main_loop, gboolean show_main_dialog)
 {
+	if (show_main_dialog)
+		gtk_widget_show (GTK_WIDGET (tool->main_dialog));
+	
 	gst_dialog_freeze_visible (tool->main_dialog);
-	gtk_widget_show (GTK_WIDGET (tool->main_dialog));
 
-	gst_tool_load_try (tool);
-	gst_dialog_thaw_visible (tool->main_dialog);
+	if (tool->remote_config == TRUE) {
+		/* run the ssh client */
+		gst_tool_fill_remote_hosts_list (tool, tool->remote_hosts);
+		gtk_widget_show (tool->remote_dialog);
+	} else {
+		/* run the su command */
+		gst_auth_do_su_authentication (tool);
+
+		gst_tool_process_startup (tool);
+		gst_dialog_apply_widget_policies (tool->main_dialog);
+
+		gst_tool_load_try (tool);
+		gst_dialog_thaw_visible (tool->main_dialog);
+	}
 
 	if (!no_main_loop)
 		gtk_main ();
 }
 
+void
+gst_tool_main (GstTool *tool, gboolean no_main_loop)
+{
+	tool_main_do (tool, no_main_loop, TRUE);
+}
+
+void
+gst_tool_main_with_hidden_dialog (GstTool *tool, gboolean no_main_loop)
+{
+	tool_main_do (tool, no_main_loop, FALSE);
+}
+
 static void
 gst_tool_destroy (GtkObject *object)
 {
@@ -1228,46 +1117,9 @@ gst_tool_class_init (GstToolClass *klass)
 			      gst_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-#if 0	
-	gtk_object_class_add_signals (object_class, gsttool_signals, LAST_SIGNAL);
-#endif	
-
 	object_class->destroy = gst_tool_destroy;
 }
 
-#if 0
-
-static void
-visibility_toggled (GtkWidget *w, gpointer data)
-{
-	GstTool *tool;
-	gboolean show;
-	GtkAdjustment *vadj;
-
-	tool = GST_TOOL (data);
-
-	show = GTK_TOGGLE_BUTTON (w)->active;
-	vadj = gtk_scrolled_window_get_vadjustment (
-		GTK_SCROLLED_WINDOW (tool->report_scrolled));
-
-	set_arrow (tool, show ? GTK_ARROW_UP : GTK_ARROW_DOWN);
-
-	if (show) {
-		gtk_widget_set_usize (tool->report_scrolled, -1, 140);
-		gtk_widget_show (tool->report_scrolled);
-		gtk_adjustment_set_value (vadj, vadj->upper - vadj->page_size);
-		gtk_adjustment_value_changed (vadj);
-	} else { 
-		gtk_widget_hide (tool->report_scrolled);
-		if (tool->timeout_done)
-			gtk_main_quit();
-	}
-
-	tool->report_list_visible = show;
-}
-
-#endif
-
 static void
 gst_tool_create_platform_list (GtkTreeView *list, GstTool *tool)
 {
@@ -1310,6 +1162,82 @@ gst_tool_create_platform_list (GtkTreeView *list, GstTool *tool)
 			  (gpointer) tool);
 }
 
+static void
+on_remote_hosts_list_selection_changed (GtkTreeSelection *selection, gpointer data)
+{
+	GstTool *tool = GST_TOOL (data);
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gchar *host;
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		/* freeze the dialog */
+		gtk_widget_set_sensitive (tool->remote_dialog, FALSE);
+		
+		gtk_tree_model_get (model, &iter, 0, &host, -1);
+
+		if (tool->backend_pid > 0) {
+			gst_dialog_ask_apply (tool->main_dialog);
+			gst_dialog_set_modified (tool->main_dialog, FALSE);
+
+			/* stops the backend */
+			gst_tool_kill_backend (tool, NULL);
+
+			/* process the "end" directive */
+			while (gtk_events_pending())
+				gtk_main_iteration();
+
+			/* remove current platform info, in the new host may be another one */
+			if (tool->current_platform != NULL) {
+				gst_platform_free (tool->current_platform);
+				tool->current_platform = NULL;
+			}
+		}
+
+		gst_dialog_freeze_visible (tool->main_dialog);
+	
+		gst_auth_do_ssh_authentication (tool, host);
+		gst_tool_process_startup (tool);
+
+		gst_tool_load_try (tool);
+		gst_dialog_apply_widget_policies (tool->main_dialog);
+
+		/* thaw all the dialogs */
+		gst_dialog_thaw_visible (tool->main_dialog);
+		gtk_widget_set_sensitive (tool->remote_dialog, TRUE);
+	}
+}
+
+static void
+gst_tool_create_remote_hosts_list (GtkTreeView *list, GstTool *tool)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_STRING));
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection *select;
+
+	g_return_if_fail (list != NULL);
+	g_return_if_fail (GTK_IS_TREE_VIEW (list));
+
+	gtk_tree_view_set_model (GTK_TREE_VIEW (list), model);
+	g_object_unref (model);
+
+	renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (list),
+						     -1,
+						     NULL,
+						     renderer,
+						     "text", 0,
+						     NULL);
+
+	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
+	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
+
+	g_signal_connect (G_OBJECT (select), "changed",
+			  G_CALLBACK (on_remote_hosts_list_selection_changed),
+			  (gpointer) tool);
+}
+
 static void
 gst_tool_type_init (GstTool *tool)
 {
@@ -1324,33 +1252,8 @@ gst_tool_type_init (GstTool *tool)
 	tool->report_window     = glade_xml_get_widget (xml, "report_window");
 	g_signal_connect (GTK_OBJECT (tool->report_window), "delete_event",
 			    G_CALLBACK (report_window_close_cb), tool);
-#if 0
-	tool->report_scrolled   = glade_xml_get_widget (xml, "report_list_scrolled_window");
-	tool->report_progress   = glade_xml_get_widget (xml, "report_progress");
-#endif
+
 	tool->report_label      = glade_xml_get_widget (xml, "report_label");
-#if 0
-	tool->report_list       = glade_xml_get_widget (xml, "report_list");
-	tool->report_entry      = glade_xml_get_widget (xml, "report_text");
-	tool->report_visibility = glade_xml_get_widget (xml, "report_visibility");
-	tool->report_notebook   = glade_xml_get_widget (xml, "report_notebook");
-
-	gtk_notebook_remove_page (GTK_NOTEBOOK (tool->report_notebook), 0);
-	gtk_notebook_append_page (GTK_NOTEBOOK (tool->report_notebook),
-				  gnome_stock_pixmap_widget_new (tool->report_notebook, 
-								 GNOME_STOCK_PIXMAP_UP),
-				  NULL);
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (tool->report_notebook),
-				  gnome_stock_pixmap_widget_new (tool->report_notebook, 
-								 GNOME_STOCK_PIXMAP_DOWN),
-				  NULL);
-
-	set_arrow (tool, GTK_ARROW_DOWN);
-	gtk_widget_show_all (tool->report_notebook);
-
-	glade_xml_signal_connect_data (xml, "visibility_toggled", visibility_toggled, tool);
-#endif
 
 	xml = gst_tool_load_glade_common (tool, "platform_dialog");
 	tool->platform_list = glade_xml_get_widget (xml, "platform_list");
@@ -1358,6 +1261,11 @@ gst_tool_type_init (GstTool *tool)
 
 	tool->platform_dialog    = glade_xml_get_widget (xml, "platform_dialog");
 	tool->platform_ok_button = glade_xml_get_widget (xml, "platform_ok_button");
+
+	xml = gst_tool_load_glade_common (tool, "remote_dialog");
+	tool->remote_dialog = glade_xml_get_widget (xml, "remote_dialog");
+	tool->remote_hosts_list = glade_xml_get_widget (xml, "remote_hosts_list");
+	gst_tool_create_remote_hosts_list (GTK_TREE_VIEW (tool->remote_hosts_list), tool);
 }
 
 GtkType
@@ -1398,15 +1306,23 @@ void
 gst_tool_construct (GstTool *tool, const char *name, const char *title)
 {
 	char *s, *t, *u;
+	GConfClient *client = gconf_client_get_default ();
+	GError *error;
+
+	gchar *remote_config_key;
+	gchar *remote_hosts_key;
+	gchar *hosts;
+	gboolean do_remote_conf;
 
 	g_return_if_fail (name != NULL);
 
 	tool->name        = g_strdup (name);
 	tool->glade_path  = g_strdup_printf ("%s/%s.glade",     INTERFACES_DIR, name);
 	tool->script_path = g_strdup_printf ("%s/%s-conf",      SCRIPTS_DIR,    name);
-	
+	tool->script_name = g_strdup_printf ("%s-conf",         name);
+
 	s = g_strdup_printf ("%s_admin", name);
-	t = g_strdup_printf (_("%s - GNOME System Tools"), title);
+	t = g_strdup_printf ("%s", title);
 	u = g_strdup_printf (PIXMAPS_DIR "/%s.png", name);
 
 	tool->main_dialog = gst_dialog_new (tool, s, t);
@@ -1421,17 +1337,33 @@ gst_tool_construct (GstTool *tool, const char *name, const char *title)
 			  "apply",
 			  G_CALLBACK (gst_tool_save_cb),
 			  tool);
+	g_signal_connect (G_OBJECT (tool->main_dialog),
+			  "restore",
+			  G_CALLBACK (gst_tool_restore_cb),
+			  tool);
 
 	tool->report_hook_list = NULL;
 	memset (&tool->report_hook_defaults, 0, sizeof (GstReportHook *) * GST_MAJOR_MAX);
 	gst_tool_add_report_hooks (tool, common_report_hooks);
 
-	tool->backend_pid = tool->backend_read_fd = tool->backend_write_fd = -1;
-	gst_tool_set_close_func (tool, gst_tool_kill_backend, NULL);
+	tool->backend_pid = -1;
+	tool->backend_master_fd = -1;
+	gst_tool_set_close_func (tool, gst_tool_kill_tool, NULL);
 
 	tool->directive_running = FALSE;
 	tool->directive_queue   = NULL;
 	tool->directive_queue_idle_id = 0;
+
+	/* get the remote configuration gconf keys */
+	remote_config_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "remote-configuration", NULL);
+	remote_hosts_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "remote-hosts-list", NULL);
+
+	do_remote_conf = gconf_client_get_bool (client, remote_config_key, &error);
+	hosts = gconf_client_get_string (client, remote_hosts_key, &error);
+
+	tool->remote_config = ((do_remote_conf) && (hosts != NULL) && (strcmp (SSH_PATH, "") != 0));
+	if (hosts != NULL) 
+		tool->remote_hosts = g_strsplit (hosts, ",", -1);
 }
 
 GstTool *
@@ -1564,31 +1496,6 @@ gst_tool_reset_report_hooks (GstTool *tool)
 		((GstReportHook *) list->data)->invoked = FALSE;
 }
 
-/* it does authentication, first of all it runs su (just to ensure that it's completely run
- * when password is sent), then asks for password and sends it to su (if you want to run it
- * with root privileges)
- */
-static void
-authenticate (int argc, char *argv[])
-{
-	gchar *password;
-	gint result;
-
-	gst_su_run_term (argc, argv, NULL);
-
-	result = gst_su_get_password (&password);
-
-	if (result == 1) 
-		gst_su_write_password (password);
-	else if (result == -1)
-		exit (0);
-	else
-		gst_su_clear_term ();
-
-	if (strlen (password) > 0)
-		memset (password, 0, strlen (password));
-}
-
 static void
 try_show_usage_warning (void)
 {
@@ -1597,14 +1504,14 @@ try_show_usage_warning (void)
 	gchar *version;
 	GConfClient *client;
 	GError *error = NULL;
-	gchar *warning = g_strdup_printf(_("Welcome to the %s prerelease of the "
-		  "GNOME System Tools.\n\n"
-		  "This is still a work in progress, and so it may have serious bugs.\n"
-		  "Due to the nature of these tools, bugs may render your computer\n"
-		  "PRACTICALLY USELESS, costing time, effort and sanity points.\n\n"
-		  "You have been warned. Thank you for trying out this prerelease of\n"
-		  "the GNOME System Tools!\n\n"
-		  "--\nThe GNOME System Tools team"), VERSION);
+	gchar *warning_title = g_strdup_printf (_("<span weight=\"bold\" size=\"larger\">"
+						  "Welcome to the %s prerelease of the GNOME System Tools"
+						  "</span>"), VERSION);
+	gchar *warning = _("This is still a work in progress, and so it may have serious bugs. "
+			   "Due to the nature of these tools, bugs may render your computer "
+			   "<span weight=\"bold\">practically useless</span>, costing time, effort and sanity points. "
+			   "You have been warned. Thank you for trying out this prerelease of "
+			   "the GNOME System Tools!");
 
 	client = gconf_client_get_default ();
 
@@ -1615,45 +1522,64 @@ try_show_usage_warning (void)
 	value = gconf_client_get_bool (client, key, &error);
 	version = gconf_client_get_string (client, version_key, &error);
 
-	if ((value == TRUE) || (strcmp (version, VERSION) != 0))
+	if ((value == TRUE) || version && (strcmp (version, VERSION) != 0))
 	{
-		GtkWidget *dialog, *label, *image, *hbox;
+		GtkWidget *dialog, *label, *title_label, *image, *hbox, *vbox;
 		GtkWidget *checkbox;
 		gboolean dont_ask_again;
 		
 		dialog = gtk_dialog_new_with_buttons (_("Warning"),
 						      NULL,
-						      GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
 						      GTK_STOCK_OK, GTK_RESPONSE_OK,
 						      NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);		
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+		title_label = gtk_label_new (warning_title);
+		gtk_label_set_line_wrap (GTK_LABEL (title_label), TRUE);
+		gtk_label_set_use_markup (GTK_LABEL (title_label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (title_label), 0, 0);
 		
 		label = gtk_label_new (warning);
+		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (title_label), 0, 0);
+		
 		image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
+		gtk_misc_set_alignment (GTK_MISC (image), 0, 0);
+		
 		checkbox = gtk_check_button_new_with_label (_("Don't show me this again"));
 
-		hbox = gtk_hbox_new (FALSE, 5);
+		hbox = gtk_hbox_new (FALSE, 12);
+		vbox = gtk_vbox_new (FALSE, 12);
 
-		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 5);
-		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
+		gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
 
-		gtk_window_set_default_size (GTK_WINDOW (dialog), 300, 150);
-		
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 5);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), checkbox, FALSE, FALSE, 5);
+		gtk_box_pack_start (GTK_BOX (vbox), title_label, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox), checkbox, FALSE, FALSE, 0);
+
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
+
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 0);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 		
 		gtk_widget_show_all (dialog);
+		
 		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
 			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)) == TRUE) {
 				gconf_client_set_bool (client, key, FALSE, &error);
 				gconf_client_set_string (client, version_key, VERSION, &error);
 			}
 		}
+		
 		gtk_widget_hide (dialog);
 		gtk_widget_destroy (dialog);
 	}
 
-	g_free (warning);
+	g_free (warning_title);
 	g_free (key);
 }
 
@@ -1688,7 +1614,6 @@ gst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 	
 		for (i = 0; args && args[i]; i++) {
 			args_list = g_list_append (args_list, args[i]);
-			g_print ("-->%s<--\n", (gchar *) args_list->data);
 		}
 		poptFreeContext (ctx);
 	}	
@@ -1699,18 +1624,68 @@ gst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 				      GNOME_PARAM_HUMAN_READABLE_NAME,
 				      _("GNOME System Tools"),
 				      NULL);
-	
-	if (geteuid () == 0) {
-		root_access = ROOT_ACCESS_REAL;
-#ifdef GST_DEBUG
-	} else if (getenv ("SET_ME_UP_HARDER")) {
-		g_warning (_("Pretending we are root..."));
-		root_access = ROOT_ACCESS_SIMULATED;
-#endif
-	} else {
-		authenticate (argc, argv);
-		root_access = ROOT_ACCESS_NONE;
-	}
-
 	try_show_usage_warning ();
 }
+
+gchar*
+gst_tool_read_from_backend (GstTool *tool)
+{
+	gchar *buffer;
+	GString *str = g_string_new ("");
+	gchar *ptr;
+	gboolean may_exit = FALSE;
+	
+	buffer = (gchar*) g_malloc0 (sizeof (gchar) * BUFFER_SIZE);
+
+	do {
+		usleep (32000);
+		
+		while (fgets (buffer, BUFFER_SIZE, tool->backend_stream) != NULL) {
+			g_string_append (str, buffer);
+			may_exit = TRUE;
+		}
+		
+		while (gtk_events_pending ())
+			gtk_main_iteration ();
+	} while (!may_exit);
+
+	ptr = str->str;
+	g_string_free (str, FALSE);
+
+	return ptr;
+}
+
+void
+gst_tool_write_xml_to_backend (GstTool *tool, xmlDoc *doc)
+{
+	gint size;
+	xmlChar *xml;
+	gchar *string;
+
+	xmlDocDumpMemory (doc, &xml, &size);
+	string = (gchar *) xml;
+	
+	gst_tool_write_to_backend (tool, string);
+
+	xmlFree (xml);
+
+	usleep (32000);
+}
+
+void
+gst_tool_write_to_backend (GstTool *tool, gchar *string)
+{
+	gint ntotal = 0;
+	gint nread = 0;
+	gint ret;
+	gchar *p;
+
+	do {
+		ret = fputc (string [nread], tool->backend_stream);
+
+		if (ret >= 0)
+			nread++;
+	} while (nread < strlen (string));
+
+	while (fflush (tool->backend_stream) != 0);
+}
