@@ -53,10 +53,6 @@ extern void on_network_notebook_switch_page (GtkWidget *notebook,
 void filter_editable (GtkEditable *e, const gchar *text, 
 		      gint length, gint *pos, gpointer data);
 
-#define connect_editable_filter(w, r) g_signal_connect (G_OBJECT (w), "insert_text", \
-			                                  G_CALLBACK (filter_editable), \
-			                                  GINT_TO_POINTER (r))
-
 
 /* libglade callbacks */
 void on_network_notebook_switch_page (GtkWidget *notebook, 
@@ -69,8 +65,12 @@ gint update_hint (GtkWidget *w, GdkEventFocus *e, gpointer null);
 void on_connection_add_clicked (GtkWidget *w, gpointer null);
 void on_connection_delete_clicked (GtkWidget *w, gpointer null);
 void on_connection_configure_clicked (GtkWidget *w, gpointer null);
-void on_connection_activate_clicked (GtkWidget *w, gpointer null);
-void on_connection_deactivate_clicked (GtkWidget *w, gpointer null);
+void on_connection_popup_add_activate (gpointer callback_data, guint action, GtkWidget *widget);
+void on_connection_popup_configure_activate (gpointer callback_data, guint action, GtkWidget *widget);
+void on_connection_popup_delete_activate (gpointer callback_data, guint action, GtkWidget *widget);
+void on_connection_activate_button_clicked (GtkWidget*, gpointer);
+void on_connection_deactivate_button_clicked (GtkWidget*, gpointer);
+
 void on_dns_dhcp_toggled (GtkWidget *w, gpointer null);
 void on_samba_use_toggled (GtkWidget *w, gpointer null);
 void on_wins_use_toggled (GtkWidget *w, gpointer null);
@@ -86,23 +86,32 @@ gboolean callbacks_check_gateway_hook      (GstDialog *dialog, gpointer data);
 
 gboolean callbacks_tool_not_found_hook     (GstTool *tool, GstReportLine *rline, gpointer data);
 
+gboolean callbacks_button_press (GtkTreeView *treeview, GdkEventButton *event, gpointer gdata);
+
 /* connection callbacks */
-void on_connection_list_clicked (GtkWidget*, gpointer);
-void on_connection_ok_clicked (GtkWidget*, GstConnection*);
-void on_connection_cancel_clicked (GtkWidget*, GstConnection*);
-void on_connection_config_dialog_destroy (GtkWidget*, GstConnection*);
-gint on_connection_config_dialog_delete_event (GtkWidget*, GdkEvent*, GstConnection*);
-void on_connection_modified (GtkWidget*, GstConnection*);
-void on_wvlan_adhoc_toggled (GtkWidget*, GstConnection*);
-void on_ppp_update_dns_toggled (GtkWidget*, GstConnection*);
-gboolean on_ip_address_focus_out (GtkWidget*, GdkEventFocus*, GstConnection*);
-void on_ppp_autodetect_modem_clicked (GtkWidget*, GstConnection*);
+void on_connection_toggled (GtkWidget*, gchar*, gpointer);
+void on_connection_ok_clicked (GtkWidget*, gpointer);
+void on_connection_cancel_clicked (GtkWidget*, gpointer);
+gboolean on_connection_delete_event (GtkWidget*, GdkEvent*, gpointer);
+void on_connection_modified (GtkWidget*, gpointer);
+void on_ppp_update_dns_toggled (GtkWidget*, gpointer);
+gboolean on_ip_address_focus_out (GtkWidget*, GdkEventFocus*, gpointer);
+void on_ppp_autodetect_modem_clicked (GtkWidget*, gpointer);
+gchar* on_volume_format_value (GtkWidget*, gdouble, gpointer);
+
+/* DNS tab callbacks */
+void on_dns_search_add_button_clicked (GtkWidget *button, gpointer gdata);
+void on_dns_search_del_button_clicked (GtkWidget *button, gpointer gdata);
+void on_dns_search_entry_changed      (GtkWidget *entry, gpointer gdata);
+void on_dns_search_popup_del_activate (gpointer callback_data, guint action, GtkWidget *widget);
 
 /* Hosts tab callbacks */
 void on_hosts_ip_changed (GtkEditable*, gpointer);
 void on_hosts_alias_changed (GtkTextBuffer*, gpointer);
 void on_hosts_add_clicked (GtkWidget*, gpointer);
 void on_hosts_delete_clicked (GtkWidget*, gpointer);
+void on_hosts_popup_del_activate (gpointer callback_data, guint action, GtkWidget *widget);
+
 
 /* Network connection druid callbacks */
 gboolean on_network_druid_hide (GtkWidget*, gpointer);
